/* ----------------------------------------------------------------------------
*
*
*                        CLASSE ChargeImgBinaire v1.0
*
*
* Flash >= AS3
* ©Lorenzo le 03/08/2010
*
 ------------------------------------------------------------------------------ */
package lol.data{

	import flash.display.DisplayObject;
	import flash.display.Loader;
	import flash.errors.IllegalOperationError;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLLoader;
	import flash.net.URLLoaderDataFormat;
	import flash.net.URLRequest;
	import flash.utils.ByteArray;
	import flash.utils.Endian;
	import lol.event.ChargeImgBinaireEvent;



	// -----------------------------------------------------------------------------------
	// Évenements
	// -----------------------------------------------------------------------------------

	/** 
	 * Propagé pendant la progression du chargement.
	 * @eventType lol.event.ChargeImgBinaireEvent.PROGRES
	 */
	[Event(name = "progres", type = "lol.event.ChargeImgBinaireEvent.PROGRES")]

	/** 
	 * Propagé lors d'une erreur de conversion.
	 * @eventType lol.event.ChargeImgBinaireEvent.ERREUR_CONVERSION
	 */
	[Event(name = "erreurConversion", type = "lol.event.ChargeImgBinaireEvent.ERREUR_CONVERSION")]

	/** 
	 * Propagé lors d'une erreur de conversion (coupe le script).
	 * @eventType lol.event.ChargeImgBinaireEvent.ERREUR_CONVERSION_FATAL
	 */
	[Event(name = "erreurConversionFatal", type = "lol.event.ChargeImgBinaireEvent.ERREUR_CONVERSION_FATAL")]
	
	/**
	 * Evenement executé lors d'une erreur de chargement du fichier.
	 * @eventType lol.event.ChargeImgBinaireEvent.ERREUR_CHARGEMENT
	 */
	[Event(name = "erreurChargement", type = "lol.event.ChargeImgBinaireEvent.ERREUR_CHARGEMENT")]

	/** 
	 * Propagé quand tout est fini (chargement/conversion).
	 * @eventType lol.event.ChargeImgBinaireEvent.FIN
	 */
	[Event(name = "fin", type = "lol.event.ChargeImgBinaireEvent.FIN")]
	
	
	


	/**
	 * Cette classe à pour but de permettre le chargement en masse de fichier image SWF/JPG/PNG/GIF.
	 * Toutes les données des images sont placés dans le meme fichier binaire qui est construit :
	 * -nb total images
	 * -image :
	 * 	poids image
	 * 	données image
	 * 
	 * @see lol.event.ChargeImgBinaireEvent
	 *  
	 * @example
	 * <listing version="3.0">
	 * var chargeur:ChargeImgBinaire = new ChargeImgBinaire();
	 * chargeur.addEventListener(ChargeImgBinaireEvent.PROGRES, evtBinaireProgres);
	 * chargeur.addEventListener(ChargeImgBinaireEvent.ERREUR_CHARGEMENT, evtBinaireErreurChargement);
	 * chargeur.addEventListener(ChargeImgBinaireEvent.ERREUR_CONVERSION, evtBinaireErreurConversion);
	 * chargeur.addEventListener(ChargeImgBinaireEvent.FIN, evtBinaireFin);
	 * chargeur.lancer("http://localhost/data_img.php");
	 * </listing>
	 */
	public class ChargeImgBinaire extends EventDispatcher {

		// -----------------------------------------------------------------------------------
		// 
		// INITIALISATION DES MEMBRES
		// 
		// -----------------------------------------------------------------------------------


		// -----------------------------------------------------------------------------------
		// Paramètres utilisateur
		// -----------------------------------------------------------------------------------

		/** 
		 * Le tableau des DisplayObject aprés chargement.
		 */
		private var _dpo:Array = new Array();
		
		/**
		 * L'URL pointant vers le fichier binaire
		 */
		private var _url:String = "";



		// -----------------------------------------------------------------------------------
		// Paramètres classe
		// -----------------------------------------------------------------------------------
		
		/**
		 * Le Loader servant de convertisseur.
		 */
		private var _convertisseur:Loader = null;

		/**
		 * Indique si le chargement est en cours.
		 */
		private var _chargeEnCours:Boolean = false;

		/**
		 * Le nombre total d'images dans le fichier
		 */
		private var _totalImg:uint = 0;

		/**
		 * L'image courante en court de traitement (erreur ou converti)
		 */
		private var _imgCourant:uint = 0;





		// -----------------------------------------------------------------------------------
		// 
		// CONSTRUCTEUR
		// 
		// -----------------------------------------------------------------------------------

		/** 
		 * Constructeur de la classe ChargeImgBinaire
		 * 
		 */
		public function ChargeImgBinaire() {
			super();
		}





		// -----------------------------------------------------------------------------------
		// 
		// METHODES
		// 
		// -----------------------------------------------------------------------------------

		/** 
		 * Lancer le chargement des fichiers
		 *
		 * @return (Void)
		 *
		 * @throws IllegalOperationError Quand le chargement est déja en cours
		 */
		public function lancer(url:String = ""):void {
			if ( this._chargeEnCours == true ) {
				throw new IllegalOperationError("ChargeImgBinaire::lancer() -> Le chargement est déja en cours.");
				return;
			}
			
			this._totalImg = 0;
			this._dpo = new Array();
			this._imgCourant = 0;
			
			if ( url ) {
				this._url = url;
			}
			
			var loadGroupImg:URLLoader = new URLLoader();
			loadGroupImg.dataFormat = URLLoaderDataFormat.BINARY;
			loadGroupImg.addEventListener(Event.COMPLETE, evtLoadGroupImgComplete);
			loadGroupImg.addEventListener(IOErrorEvent.IO_ERROR, evtLoadGroupImgIOError);
			loadGroupImg.addEventListener(SecurityErrorEvent.SECURITY_ERROR, evtLoadGroupImgSecurityError);
			loadGroupImg.addEventListener(ProgressEvent.PROGRESS, evtLoadGroupImgProgress);
			loadGroupImg.load(new URLRequest(this._url));
			this._chargeEnCours = true;
		}





		// -----------------------------------------------------------------------------------
		// 
		// EVENEMENTS
		// 
		// -----------------------------------------------------------------------------------

		
		// ---------------------- URLLoader ----------------------
		private function evtLoadGroupImgProgress(ev:ProgressEvent):void {
			// progression du chargement des données binaire
			this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.PROGRES, false, false, (ev.bytesLoaded / ev.bytesTotal) * 100));
		}
		
		private function evtLoadGroupImgIOError(ev:IOErrorEvent):void {
			// erreur de chargement des données binaire
			var loadGroupImg:URLLoader = (ev.currentTarget as URLLoader);
			loadGroupImg.removeEventListener(Event.COMPLETE, evtLoadGroupImgComplete);
			loadGroupImg.removeEventListener(IOErrorEvent.IO_ERROR, evtLoadGroupImgIOError);
			loadGroupImg.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, evtLoadGroupImgSecurityError);
			loadGroupImg.removeEventListener(ProgressEvent.PROGRESS, evtLoadGroupImgProgress);
			this._chargeEnCours = false;
			this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.ERREUR_CHARGEMENT, false, false, 0, ev.text));
		}
		
		private function evtLoadGroupImgSecurityError(ev:SecurityErrorEvent):void {
			// erreur de chargement des données binaire
			var loadGroupImg:URLLoader = (ev.currentTarget as URLLoader);
			loadGroupImg.removeEventListener(Event.COMPLETE, evtLoadGroupImgComplete);
			loadGroupImg.removeEventListener(IOErrorEvent.IO_ERROR, evtLoadGroupImgIOError);
			loadGroupImg.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, evtLoadGroupImgSecurityError);
			loadGroupImg.removeEventListener(ProgressEvent.PROGRESS, evtLoadGroupImgProgress);
			this._chargeEnCours = false;
			this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.ERREUR_CHARGEMENT, false, false, 0, ev.text));
		}
		
		private function evtLoadGroupImgComplete(ev:Event):void {
			// chargement des données binaire réussi
			var loadGroupImg:URLLoader = (ev.currentTarget as URLLoader);
			loadGroupImg.removeEventListener(Event.COMPLETE, evtLoadGroupImgComplete);
			loadGroupImg.removeEventListener(IOErrorEvent.IO_ERROR, evtLoadGroupImgIOError);
			loadGroupImg.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, evtLoadGroupImgSecurityError);
			loadGroupImg.removeEventListener(ProgressEvent.PROGRESS, evtLoadGroupImgProgress);
			this._chargeEnCours = false;
			
			var ba:ByteArray = loadGroupImg.data;
			ba.endian = Endian.BIG_ENDIAN;
			try {
				// lire le nombre au début du fichier binaire
				this._totalImg = ba.readUnsignedInt();
			}catch (er:Error) {
				this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.ERREUR_CONVERSION_FATAL, false, false, 0, er.message));
				loadGroupImg.data = null;
				loadGroupImg = null;
				return;
			}
			
			var jpgLength:uint;
			var baJpg:ByteArray;
			var conversion:Loader;
			while (ba.position < ba.length) {
				try {
					// lecture du 'poids' de l'image courante
					jpgLength = ba.readUnsignedInt();
				}catch (er:Error) {
					this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.ERREUR_CONVERSION_FATAL, false, false, 0, er.message));
					loadGroupImg.data = null;
					loadGroupImg = null;
					return;
				}

				
				baJpg = new ByteArray();
				try {
					// ecriture des données binaires de l'image courante dans un ByteArray
					baJpg.writeBytes(ba, ba.position, jpgLength);
				}catch (er:Error) {
					this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.ERREUR_CONVERSION_FATAL, false, false, 0, er.message));
					loadGroupImg.data = null;
					loadGroupImg = null;
					return;
				}
				ba.position += jpgLength;
			
				// -------------------------------------
				conversion = new Loader();
				conversion.contentLoaderInfo.addEventListener(Event.COMPLETE, evtConversionComplete);
				conversion.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, evtConversionIOError);

				try {
					// conversion du ByteArray en DisplayObject
					conversion.loadBytes(baJpg);
				}catch (er:Error) {
					this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.ERREUR_CONVERSION, false, false, 0, er.message));
					loadGroupImg.data = null;
					loadGroupImg = null;
					return;
				}
			}
		}

		
		// ---------------------- LOADER - conversion ----------------------
		private function evtConversionIOError(ev:IOErrorEvent):void {
			// déclenché si les données binaires ne peuvent pas être recomposé
			// dans un des types supportés par flash (gif/png/jpg/swf)
			var conversion:Loader = ev.currentTarget.loader as Loader;
			conversion.contentLoaderInfo.removeEventListener(Event.COMPLETE, evtConversionComplete);
			conversion.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, evtConversionIOError);
			conversion.unload();
			conversion = null;
			this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.ERREUR_CONVERSION, false, false, 0, ev.text));
			this._imgCourant++;
			if ( this._imgCourant >= this._totalImg ) {
				this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.FIN));
			}
		}
		
		private function evtConversionComplete(ev:Event):void {
			// conversion des données binaires en un DisplayObject réussi
			var conversion:Loader = ev.currentTarget.loader as Loader;
			var dob:DisplayObject = conversion.content as DisplayObject;
			this._dpo.push(dob);
			
			conversion.contentLoaderInfo.removeEventListener(Event.COMPLETE, evtConversionComplete);
			conversion.contentLoaderInfo.removeEventListener(IOErrorEvent.IO_ERROR, evtConversionIOError);
			conversion.unload();
			conversion = null;
			
			this._imgCourant++;
			if ( this._imgCourant >= this._totalImg ) {
				this.dispatchEvent(new ChargeImgBinaireEvent(ChargeImgBinaireEvent.FIN));
			}
		}





		// -----------------------------------------------------------------------------------
		// 
		// GET / SET
		// 
		// -----------------------------------------------------------------------------------
		
		/**
		 * Le tableau des DisplayObject aprés chargement.
		 */
		public function get images():Array { return _dpo; }
		

		/**
		 * L'URL pointant vers le fichier binaire.
		 */
		public function get url():String { return _url; }
		public function set url(value:String):void {
			_url = value;
		}
		
		/**
		 * Le total des images chargés (nombre lu au début du fichier).
		 */
		public function get totalImg():uint { return _totalImg; }
	}
}